
import requests
from collections import namedtuple

ControlVoc = namedtuple("ControlVoc", ["ID", "short", "long"])

class countryCodes:
    """! this is a quick and dirty utility class to obtain all valid values of a specific metadata value of the stations
    It is created with the example of country codes in mind, to split the number of requests to the TOAD database for each specific analysis request.
    Argument:
    ---------
    controlVocEndpoint:
        endpoint to access all valid values of a variable used within the control vocabulary
    stationMetaEndpoint:
        endpoint to access all stations fitting to a specific search pattern. Will be used to validate, that a metadata values provides access to stations.
    """
    def __init__(self, controlVocEndpoint="https://toar-data.fz-juelich.de/api/v2/controlled_vocabulary/", stationMetaEndpoint="https://toar-data.fz-juelich.de/api/v2/stationmeta/"):
        self.controlVocEndpoint = controlVocEndpoint
        if self.controlVocEndpoint[-1] != "/":
            self.controlVocEndpoint += "/"
        self.stationMetaEndpoint = stationMetaEndpoint
        if self.stationMetaEndpoint[-1] != "/":
            self.stationMetaEndpoint += "/"
    def getValidVocabular(self, controlName, varName ):
        """get all valid values for a variable
        Argument:
        --------
        controlName:
            name of the parameter in the control vocabulary
        varName:
            name of the parameter in a request to another endpoint. 
        return:
            list with all valid values for the requested varName to be used within request to the TOAR database
        """
        #get all possible values for the given variable:
        response  = requests.get( f"{self.controlVocEndpoint}{controlName}")
        response.raise_for_status()
        controlVoc = self.convertLists(response)
        #check for all valid infos, i.e. is there at least one station for this metadata
        validCodes = []
        for voc in controlVoc:
            params = {varName : voc.short, "limit" : 1}
            getNumber = requests.get(self.stationMetaEndpoint, params=params  )
            getNumber.raise_for_status()
            if len(getNumber.json()) > 0:
                validCodes.append(voc.short)
        return validCodes
            
    def convertLists(self, response) -> list[namedtuple]:
        """convert results into an easier to access data type
        """
        return [  ControlVoc(id, short, long)  for id, short, long in response.json()]
